/*
The following software may be included in this product: D3.js.

A copy of the source code may be downloaded from https://github.com/d3/d3. This software contains the following license and notice below:
Copyright 2010-2023 Mike Bostock

Permission to use, copy, modify, and/or distribute this software for any purpose with or without fee is hereby granted, provided that the above copyright notice and this permission notice appear in all copies.
THE SOFTWARE IS PROVIDED “AS IS” AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

*/
var BarGauge = /** @class */ (function () {
    function BarGauge(selector, width, height) {
        this.formatter = d3.format(".0%"); //see https://devdocs.io/d3~4/d3-format
        if(this.value === undefined)
			this.value = 0;
        this.width = width;
        this.height = height;
        this.svg = d3.select(selector).append("svg");
        this.svg.attr("width", this.width).attr("height", this.height);
        this.outer = this.svg.append("rect");
        this.outer.attr("id", "outer").attr("rx", "3").attr("stroke", "black");
        this.filler = this.svg.append("rect");
        this.filler.attr("id", "filler").attr("rx", "3").attr("stroke", "black");
        this.axis = this.svg.append("g");
        this.axis.attr("id", "axis");
		if(this.color === undefined)
			this.color = "#4287f5";
		if(this.fill_color === undefined)
			this.fill_color = "#42f542";
		if(this.minimum === undefined)
			this.minimum = 0;
		if(this.maximum === undefined)
			this.maximum = 100;
        this.isVertical = !(width > height);
        this.isScaleBeforeElement = false;
        this.scale = d3.scaleLinear();
        this.Update();
    }
    BarGauge.prototype.SetMaximum = function (maximum) {
        this.maximum = maximum;
		this.Update();
    };
	
	
    BarGauge.prototype.SetMinimum = function (minimum) {
        this.minimum = minimum;
		this.Update();
    };
	
    BarGauge.prototype.SetColor = function (color) {
        this.color = color;
        if (typeof this.svg != "undefined")
            this.outer.attr("fill", this.color);
		this.Update();
    };
	
    BarGauge.prototype.SetFillColor = function (fill_color) {
        this.fill_color = fill_color;
        if (typeof this.svg != "undefined")
            this.filler.attr("fill", this.fill_color);
		this.Update();
    };
	
    BarGauge.prototype.SetScalePos = function (scale_before) {
        this.isScaleBeforeElement = scale_before;
		this.Update();
    };
	
	BarGauge.prototype.SetSize = function(width, height)
	{
		this.width = width;
		this.height = height;
	};
	
    BarGauge.prototype.Update = function (updated) {
        if (typeof this.svg != "undefined") {
            if (this.isScaleBeforeElement != this.isScaleBeforeElementPrev) {
                this.axis.remove();
                this.axis = this.svg.append("g");
                this.axis.attr("id", "axis");
            }
            var x1 = void 0, x2 = void 0, y1 = void 0, y2 = void 0, w1 = void 0, w2 = void 0, h1 = void 0, h2 = void 0;
            if (this.isVertical) {
                y1 = 0;
                h1 = this.height;
                y2 = this.height - 8;
                h2 = 0;
                w2 = this.width - 48;
                this.axis_y = 4;
                this.scale.domain([1.0, 0.0]);
                this.scale.range([0, y2]);
                if (this.isScaleBeforeElement) {
                    //outer
                    x1 = 38;
                    w1 = this.width - 42;
                    //filler
                    x2 = 40 + 1;
                    //axis
                    this.axis_x = x1 - 3;
                    this.d3_axis = d3.axisLeft();
                }
                else {
                    //outer
                    x1 = 2;
                    w1 = this.width - 40;
                    //filler
                    x2 = 6;
                    //axis
                    this.axis_x = this.width - 36;
                    this.d3_axis = d3.axisRight();
                }
            }
            else {
                this.scale.domain([0.0, 1.0]);
                this.axis_x = 1;
                x1 = 4;
                x2 = 8;
                w1 = this.width - 14;
                h1 = this.height - 40;
                w2 = this.width - 16;
                h2 = this.height - 48;
                this.scale.range([x2, w2]);
                if (this.isScaleBeforeElement) {
                    //outer
                    y1 = 30;
                    //filler
                    y2 = 34;
                    //axis
                    this.axis_y = y1 - 3;
                    this.d3_axis = d3.axisTop();
                }
                else {
                    //outer
                    y1 = 0;
                    //filler
                    y2 = 4;
                    //axis
                    this.axis_y = this.height - 38;
                    this.d3_axis = d3.axisBottom();
                }
            }
            this.d3_axis.scale(this.scale).tickFormat(this.formatter);
            this.axis
                .attr("transform", "translate(" + this.axis_x + "," + this.axis_y + ")")
                .attr("id", "axis")
                .call(this.d3_axis);
            this.outer
                .attr("x", x1)
                .attr("y", y1)
                .attr("width", w1)
                .attr("height", h1)
                .attr("fill", this.color);
            this.filler
                .attr("x", x2)
                .attr("y", y2)
                .attr("width", w2)
                .attr("height", h2)
                .attr("fill", this.fill_color);
			if(this.value !== undefined && !updated)
				this.SetValue(this.value, false);
        }
    };
    BarGauge.prototype.SetValue = function (value, update) {
        if (typeof this.svg != "undefined") {
			if(update)
				this.Update(true);
            if (value < this.minimum)
                value = this.minimum;
            if (value > this.maximum)
                value = this.maximum;
            this.value = value;
            var diff = this.maximum - this.minimum;
            value = (value - this.minimum) / diff;
            if (this.isVertical) {
                var h = (this.height - 8) * value;
                this.filler
                    .attr("y", this.height - 4 - h)
                    .attr("height", h);
            }
            else {
                var w = (this.width - 24) * value;
                this.filler.attr("width", w);
            }
        }
    };
    return BarGauge;
}());
