/*
The following software may be included in this product: D3.js.

A copy of the source code may be downloaded from https://github.com/d3/d3. This software contains the following license and notice below:
Copyright 2010-2023 Mike Bostock

Permission to use, copy, modify, and/or distribute this software for any purpose with or without fee is hereby granted, provided that the above copyright notice and this permission notice appear in all copies.
THE SOFTWARE IS PROVIDED “AS IS” AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

*/

var DonutGauge = /** @class */ (function () {
    function DonutGauge(selector, width, height) {
        this.minimum = 0;
        this.maximum = 100;
        this.margin = { Left: 5, Right: 5, Top: 8, Bottom: 8 };
        this.start_angle = -Math.PI + 0.1 * Math.PI;
        this.end_angle = Math.PI - 0.1 * Math.PI;
        this.DeviderColor = "Black";
        //Labels
        this.LabelTopText = "";
        this.LabelBottomText = "";
        this.start_angle_slider = 112;
        this.end_angle_slider = 67.5;
        this.width = width;
		this.dragEventCount = 0;
        this.height = height;
        this.svg = d3.select(selector).append("svg");
        this.svg.attr("height", this.height).attr("width", this.width);
        this.radius = (Math.ceil(Math.min(height / 2, (width / 2))) - 10);
        this.innerRadius = this.radius - 20;
        this.outerRadius = this.radius;
        this.centerX = (this.margin.Left + this.outerRadius);
        this.centerY = (this.margin.Top + this.outerRadius);
		this.targetValueOut = 0;
        this.svg
            .append("g")
            .attr("transform", "translate(" +
            this.centerX +
            "," +
            this.centerY +
            ")");
        this.outer_path = this.svg
            .append("path").attr("transform", "translate(" +
            this.centerX +
            "," +
            this.centerY +
            ")");
        this.path = this.svg
            .append("path").attr("transform", "translate(" +
            this.centerX +
            "," +
            this.centerY +
            ")");
        this.outer_arc = d3.arc().innerRadius(this.innerRadius - 2)
            .outerRadius(this.outerRadius + 2).startAngle(this.start_angle - 0.01).endAngle(this.end_angle + 0.01);
        //Label Text
        this.svg.append("text").attr("class", "LabelTop").style("text-anchor", "middle");
        this.svg.append("text").attr("class", "LabelBottom").style("text-anchor", "middle");
        //Devider
        var pathOfDevider = d3.line()([
            [this.radius - this.radius / 4, this.radius],
            [this.radius + this.radius / 4, this.radius]
        ]);
        this.svg.append('path').attr("class", "devider").attr('d', pathOfDevider).attr("fill", "none").attr("stroke", this.DeviderColor);
        this.sliderRadius = (this.outerRadius - this.innerRadius) / 2;
        //mouse
        this.slider = this.svg.append("circle")
            .attr("r", this.sliderRadius)
            .style("fill", this.SliderColor)
            .style("cursor", "grab");
        var inst = this;
		
        var drag = d3.drag().on("drag", function (event) {
			var x = d3.event.x;
			var y = d3.event.y;
            var angle = inst.calculateAngle(x, y);
            if (!(angle < inst.start_angle_slider && angle > inst.end_angle_slider)) {
                inst.updateCirclePosition(angle);
            }
            else {
                var d1 = inst.start_angle_slider - angle;
                var d2 = angle - inst.end_angle_slider;
                if (d1 > d2) {
                    angle = inst.end_angle_slider;
                    inst.updateCirclePosition(angle);
                }
                else if (d2 > d1) {
                    angle = inst.start_angle_slider;
                    inst.updateCirclePosition(angle);
                }
            }
			
			inst.targetValueOut = inst.calculateTargetValue(angle);
			inst.dragEventCount++;
			if(inst.dragEventCount>15)
			{
				inst.dragEventCount = 0;
				var evt = new CustomEvent("UpdateTargetValue", {detail: parseInt(inst.targetValueOut)});
				window.dispatchEvent(evt);
			}
            d3.selectAll(".target").text("Target:" + inst.targetValueOut);
			
        }).on("start", function(event)
		{
			inst.dragEventCount = 0;
		}).on("end", function(event)
		{
			var evt = new CustomEvent("UpdateTargetValue", {detail: parseInt(inst.targetValueOut)});
			window.dispatchEvent(evt);
		});		
		
        this.slider.call(drag);
        //https://stackoverflow.com/questions/12277776/how-to-add-drop-shadow-to-d3-js-pie-or-donut-chart
        var defs = this.svg.append("defs");
        var filter = defs.append("filter").attr("id", "dropshadow");
        filter.append("feGaussianBlur")
            .attr("in", "SourceAlpha")
            .attr("stdDeviation", 4)
            .attr("result", "blur");
        filter.append("feOffset")
            .attr("in", "blur")
            .attr("dx", 2)
            .attr("dy", 2)
            .attr("result", "offsetBlur");
        var feMerge = filter.append("feMerge");
        feMerge.append("feMergeNode").attr("in", "offsetBlur");
        feMerge.append("feMergeNode").attr("in", "SourceGraphic");
    }
    DonutGauge.prototype.scale = function (value, value_min, value_max, min, max) {
        return (value - value_min) * (max - min) / (value_max - value_min) + min;
    };
    DonutGauge.prototype.calculateTargetValue = function (angle) {
        angle -= this.start_angle_slider;
        if (angle < 0)
            angle += 360;
        var max = 360 - (this.start_angle_slider - this.end_angle_slider);
        var min = 0;
        return this.scale(angle, min, max, this.minimum, this.maximum);
    };
    DonutGauge.prototype.SetTargetValue = function (value) {
        var max = 360 - (this.start_angle_slider - this.end_angle_slider);
        var angle = this.scale(value, this.minimum, this.maximum, 0, max);
        this.updateCirclePosition(this.start_angle_slider + angle);
    };
    DonutGauge.prototype.calculateAngle = function (x, y) {
        var angleRad = Math.atan2(y - this.centerY, x - this.centerX);
        var angleDeg = (angleRad * 180) / Math.PI;
        angleDeg = (angleDeg + 360) % 360;
        return angleDeg;
    };
    DonutGauge.prototype.updateCirclePosition = function (angle) {
        var rad = this.innerRadius / 4;
        var circleX = this.centerX + (this.innerRadius + rad - 2.5 * this.sliderRadius + 2) * Math.cos((angle * Math.PI) / 180);
        var circleY = this.centerY + (this.innerRadius + rad - 2.5 * this.sliderRadius + 2) * Math.sin((angle * Math.PI) / 180);
        this.slider.attr("cx", circleX).attr("cy", circleY);
    };
    DonutGauge.prototype.getPerctentage = function (min, max, value) {
        if (value < min)
            return min;
        if (value > max)
            return max;
        if (min == max)
            return 0;
        return (value - min) / (max - min);
    };
    DonutGauge.prototype.SetMinimum = function (min) {
        this.minimum = min;
    };
    DonutGauge.prototype.SetMaximum = function (max) {
        this.maximum = max;
    };
    DonutGauge.prototype.SetForegroundColor = function (col) {
        this.BarForegroundColor = col;
    };
    DonutGauge.prototype.SetBackgroundColor = function (col) {
        this.BarBackgroundColor = col;
    };
    DonutGauge.prototype.SetSliderColor = function (col) {
        this.SliderColor = col;
        this.slider.style("fill", this.SliderColor);
    };
    DonutGauge.prototype.SetDeviderColor = function (col) {
        this.DeviderColor = col;
        this.svg.selectAll(".devider").attr("stroke", this.DeviderColor);
    };
    DonutGauge.prototype.SetLabelTopFont = function (font) {
        this.LabelTopFont = font;
        this.UpdateLabelTop();
    };
    DonutGauge.prototype.SetLabelBottomFont = function (font) {
        this.LabelBottomFont = font;
        this.UpdateLabelBottom();
    };
    DonutGauge.prototype.SetValueFont = function (font) {
        this.ValueFont = font;
    };
    DonutGauge.prototype.SetLabelTopText = function (s) {
        this.LabelTopText = s;
        this.UpdateLabelTop();
    };
    DonutGauge.prototype.SetLabelBottomText = function (s) {
        this.LabelBottomText = s;
        this.UpdateLabelBottom();
    };
    DonutGauge.prototype.SetAllowChange = function (b) {
        console.log(b);
        this.isChangeable = b;
        this.slider.attr("opacity", b ? 1 : 0);
    };
    DonutGauge.prototype.UpdateLabelTop = function () {
        if (typeof this.LabelTopFont != undefined)
            this.svg.select(".LabelTop").attr("transform", "translate(" +
                this.centerX +
                "," +
                (this.radius - 8) +
                ")").text(this.LabelTopText)
                .attr("font-size", this.LabelTopFont.Size)
                .attr("font-family", this.LabelTopFont.Name)
                .style('fill', this.LabelTopFont.Color);
    };
    DonutGauge.prototype.UpdateLabelBottom = function () {
        if (typeof this.LabelBottomFont != undefined)
            this.svg.select(".LabelBottom").attr("transform", "translate(" +
                this.centerX +
                "," +
                (this.radius * 2 - 8) +
                ")").text(this.LabelBottomText)
                .attr("font-size", this.LabelBottomFont.Size)
                .attr("font-family", this.LabelBottomFont.Name)
                .style('fill', this.LabelBottomFont.Color);
    };
    DonutGauge.prototype.SetValue = function (i) {
        if (this.minimum == undefined || this.maximum == undefined)
            return;
        var perc = this.getPerctentage(this.minimum, this.maximum, i);
        var angle = this.start_angle + (this.end_angle - this.start_angle) * perc;
        var arc = d3.arc().innerRadius(this.innerRadius)
            .outerRadius(this.outerRadius).startAngle(this.start_angle).endAngle(angle);
        this.outer_path
            .attr("d", this.outer_arc.cornerRadius(20))
            .attr("class", "graph")
            .attr("fill", this.BarBackgroundColor).attr("filter", "url(#dropshadow)");
        this.path
            .attr("d", arc.cornerRadius(20))
            .attr("class", "graph")
            .attr("fill", this.BarForegroundColor).attr("filter", "url(#dropshadow)");
        //Current Value Label
        this.svg.selectAll(".value_label").remove();
		
		//hack bis xyz umgesetzt wurde
		if(this.ValueFont == undefined)
		{
			this.ValueFont = 
			{
				Size: 9,
				Name: "Arial",
				Color: "black",
			};
		}	

		
        this.svg.append("text").attr("transform", "translate(" +
            this.centerX +
            "," +
            (this.radius + 10 + this.ValueFont.Size / 2) +
            ")").attr("class", "value_label").style('fill', this.ValueColor)
            .attr("font-size", this.ValueFont.Size)
            .attr("font-family", this.ValueFont.Name)
            .style("text-anchor", "middle")
            .style('fill', this.ValueFont.Color)
            .text(i.toString() + "/" + this.maximum.toString());
    };
    return DonutGauge;
}());
